<?php
/**
 * @version    0.0.1
 * @package    com_ganewsletter
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Ganewsletter\Site\Model;

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\MVC\Model\ItemModel;
use \Joomla\CMS\Helper\TagsHelper;
use \GlennArkell\Component\Ganewsletter\Administrator\Helper\GanewsletterHelper;
use \GlennArkell\Component\Ganewsletter\Administrator\Helper\GanewsrptHelper;

/**
 * Item model.
 * @since  1.6
 */
class NewsletterModel extends ItemModel
{
    public $_item;

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	protected function populateState()
	{
		$app  = Factory::getApplication('com_ganewsletter');
		$user = GanewsletterHelper::getSpecificUser();

		// Check published state
		if ((!$user->authorise('core.edit.state', 'com_ganewsletter')) && (!$user->authorise('core.edit', 'com_ganewsletter')))
		{
			$this->setState('filter.published', 1);
			$this->setState('filter.archived', 2);
		}

		// Load state from the request userState on edit or from the passed variable on default
		if (Factory::getApplication()->input->get('layout') == 'edit')
		{
			$id = Factory::getApplication()->getUserState('com_ganewsletter.edit.newsletter.id');
		}
		else
		{
			$id = Factory::getApplication()->input->get('id');
			Factory::getApplication()->setUserState('com_ganewsletter.edit.newsletter.id', $id);
		}

		$this->setState('newsletter.id', $id);

		// Load the parameters.
		$params       = $app->getParams();
		$params_array = $params->toArray();

		if (isset($params_array['item_id'])) {
			$this->setState('newsletter.id', $params_array['item_id']);
		}

		$this->setState('params', $params);
	}

	/**
	 * Method to get an object.
	 * @param   integer $id The id of the object to get.
	 * @return  mixed    Object on success, false on failure.
     * @throws Exception
	 */
	public function getItem($id = null)
	{
        if ($this->_item === null) {
            $this->_item = false;

            if (empty($id)) {
                $id = $this->getState('newsletter.id');
            }

            // Get a level row instance.
            $table = $this->getTable();

            // Attempt to load the row.
            if ($table->load($id)) {

                // Check published state.
                if ($published = $this->getState('filter.published')) {
                    if (isset($table->state) && $table->state != $published) {
                        throw new \Exception(Text::_('COM_GANEWSLETTER_ITEM_NOT_LOADED'), 403);
                    }
                }

                // Convert the JTable to a clean JObject.
                $properties  = $table->getProperties(1);
                $this->_item = ArrayHelper::toObject($properties, 'JObject');

            }

            if (empty($this->_item)) {
				throw new \Exception(Text::_('COM_GANEWSLETTER_ITEM_NOT_LOADED'), 404);
			}
        }

		if (isset($this->_item->created_by)) {
			$this->_item->created_by_name = GanewsletterHelper::getSpecificUser($this->_item->created_by)->name;
		}

		if (isset($this->_item->modified_by)) {
			$this->_item->modified_by_name = GanewsletterHelper::getSpecificUser($this->_item->modified_by)->name;
		}
		if (isset($this->_item->article_list)) {
			$this->_item->article_list = json_decode($this->_item->article_list);
		}
		if (isset($this->_item->article_list)) {
            $data = '';
            $article_list = json_decode($this->_item->article_list);
            foreach ($article_list as $article) {
                $data .= GanewsletterHelper::getNewsArticle($article)->title.'<br />';
            }
            $this->_item->article_list = $data;
		}

        return $this->_item;
    }

	/**
	 * Get an instance of Table class
	 * @param   string $type   Name of the Table class to get an instance of.
	 * @param   string $prefix Prefix for the table class name. Optional.
	 * @param   array  $config Array of configuration values for the JTable object. Optional.
	 * @return  JTable|bool JTable if success, false on failure.
	 */
	public function getTable($type = 'Newsletter', $prefix = 'Administrator', $config = array())
	{
		return parent::getTable($type, $prefix, $config);
	}

	/**
	 * Get the id of an item by alias
	 * @param   string $alias Item alias
	 * @return  mixed
	 */
	public function getItemIdByAlias($alias)
	{
        $table      = $this->getTable();
        $properties = $table->getProperties();
        $result     = null;
        $aliasKey   = null;
        if (method_exists($this, 'getAliasFieldNameByView')) {
            $aliasKey   = $this->getAliasFieldNameByView('newsletter');
        }

        if (key_exists('alias', $properties)) {
            $table->load(array('alias' => $alias));
            $result = $table->id;
        } elseif (isset($aliasKey) && key_exists($aliasKey, $properties)) {
            $table->load(array($aliasKey => $alias));
            $result = $table->id;
        }

        return $result;

	}

	/**
	 * Method to check in an item.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkin($id = null)
	{
		// Get the id.
		$id = (!empty($id)) ? $id : (int) $this->getState('newsletter.id');
                
		if ($id) {
			// Initialise the table
			$table = $this->getTable();

			// Attempt to check the row in.
			if (method_exists($table, 'checkin')) {
				if (!$table->checkin($id)) {
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Method to check out an item for editing.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkout($id = null)
	{
		// Get the user id.
		$id = (!empty($id)) ? $id : (int) $this->getState('newsletter.id');

                
		if ($id) {
			// Initialise the table
			$table = $this->getTable();

			// Get the current user object.
			$user = GanewsletterHelper::getSpecificUser();

			// Attempt to check the row out.
			if (method_exists($table, 'checkout')) {
				if (!$table->checkout($user->id, $id)) {
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Publish the element
	 * @param   int $id    Item id
	 * @param   int $state Publish state
	 * @return  boolean
	 */
	public function publish($id, $state)
	{
		$table = $this->getTable();
                
		$table->load($id);
		$table->state = $state;
		$table->article_list = json_decode($table->article_list);

		return $table->store();
                
	}

	/**
	 * Method to delete an item
	 * @param   int $id Element id
	 * @return  bool
	 */
	public function delete($id)
	{
		$table = $this->getTable();
		$table->load($id);
		$table->state = -2;
		$table->article_list = json_decode($table->article_list);

		return $table->store();
	}
	
	/**
	 * Method to delete an item
	 * @param   int $id Element id
	 * @param   object $item newletter record
	 * @return  bool
	 */
	public function genPdf($id, $item)
	{
		$rptOk = false;
        $attachfile = GanewsrptHelper::createNewsReport($id, $item);

        if ($attachfile) {
            $date = GanewsletterHelper::getTodaysDate();
            $today = date_format($date,'Y-m-d H:i:s');
    		$table = $this->getTable();
    		$table->load($id);
    		$table->pdf_gen = 1;
    		$table->publish_date = $today;
    		$table->article_list = json_decode($table->article_list);

    		$table->store();
    		
            $rptOk = GanewsrptHelper::despatchNews($item, $attachfile);
		}

		return $rptOk;
	}

	/**
	 * Method to send newsletter
	 * @param   int $id Element id
	 * @param   object $item newletter record
	 * @return  bool
	 */
	public function sendNews($id, $item)
	{
		$rptOk = false;
        $attachfile = GanewsrptHelper::createNewsReport($id, $item);
        $rptOk = GanewsrptHelper::despatchNews($item, $attachfile);

		return $rptOk;
	}

}
