<?php
/**
 * @version    0.0.1
 * @package    com_ganewsletter
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Uri\Uri;
use \GlennArkell\Component\Ganewsletter\Administrator\Helper\GanewsletterHelper;

// Load admin language file
$lang = Factory::getLanguage();
$lang->load('com_ganewsletter', JPATH_ADMINISTRATOR);

/** @var \Joomla\CMS\WebAsset\WebAssetManager $wa */
$this->document->getWebAssetManager()
    ->usePreset('com_ganewsletter.ganewsletterpreset');

$user = GanewsletterHelper::getSpecificUser();
$canEdit = GanewsletterHelper::canUserEdit($user, $this->item);

$canDelete = $user->authorise('core.delete','com_ganewsletter.newsletter.'.$this->item->id);
$newsData = GanewsletterHelper::getNewsletterRecord($this->item->id);
echo '<pre>Test<br />';
print_r($this->params->get('news_loc'));
echo '</pre>';
/*
print_r(Factory::getApplication()->getUserState('com_ganewsletters.test.data'));
*/

// Display a heading for the page
if ($this->params->get('page_title', '') > '') {
	echo '<h1>'.$this->params->get('page_title').'</h1>';
} else { 
	echo '<h1>'.Text::_('COM_GANEWSLETTER_TITLE_NEWSLETTER').'</h1>';
}
?>

<div class="item_fields">

	<table class="table">

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_NEWSLETTER_TITLE'); ?></th>
			<td><?php echo $this->item->title; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_NEWSLETTER_ARTICLE_LIST'); ?></th>
			<td><?php echo $this->item->article_list; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_CREATED_DATE'); ?></th>
			<td><?php echo $this->item->created_date; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_MODIFIED_DATE'); ?></th>
			<td><?php echo $this->item->modified_date; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_NEWSLETTER_PUBLISH_DATE'); ?></th>
			<td><?php 
					$tdate = $this->item->publish_date;
					echo $tdate > 0 ? HTMLHelper::_('date', $tdate, Text::_('COM_GANEWSLETTER_DISPLAY_DATE')) : '-';
				?>
			</td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GANEWSLETTER_FORM_LBL_COMMENT'); ?></th>
			<td><?php echo nl2br($this->item->comment); ?></td>
		</tr>

	</table>

</div>

<?php /* ----------------------   Return button   ------------------------------ */ ?>
<a class="btn btn-secondary" href="<?php echo Route::_('index.php?option=com_ganewsletter&view=newsletters'); ?>">
	<i class="icon-undo"></i> <?php echo Text::_("COM_GANEWSLETTER_RETURN"); ?>
</a>

<?php /* ----------------------   Edit button   ------------------------------ */ ?>
<?php if($canEdit && $this->item->checked_out == 0): ?>
	<a class="btn btn-warning"
		href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.edit&id='.$this->item->id); ?>">
		<i class="icon-edit"></i> <?php echo Text::_("COM_GANEWSLETTER_EDIT_ITEM"); ?>
	</a>
<?php endif; ?>

<?php /* ----------------------   Delete button and modal  ------------------------------ */ ?>
<?php if ($canDelete) : ?>
	<button class="btn btn-danger w5rem mb-1" data-bs-target="#deleteModal" data-bs-toggle="modal">
		<i class="icon-trash"></i> <?php echo Text::_("COM_GANEWSLETTER_DELETE_ITEM"); ?>
	</button>

	<?php HTMLHelper::_('bootstrap.renderModal', 'deleteModal'); ?>

	<div id="deleteModal" class="modal hide fade" tabindex="-1" role="dialog" aria-labelledby="deleteModal" aria-hidden="true">
		<div class="modal-header">
			<button type="button" class="close" data-bs-dismiss="modal" aria-hidden="true">&times;</button>
			<h3><?php echo Text::_('COM_GANEWSLETTER_DELETE_ITEM'); ?></h3>
		</div>
		<div class="modal-body">
			<p><?php echo Text::sprintf('COM_GANEWSLETTER_DELETE_CONFIRM', $this->item->id); ?></p>
		</div>
		<div class="modal-footer">
			<button class="btn" data-bs-dismiss="modal">Close</button>
			<a class="btn btn-danger"
				href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.remove&id=' . $this->item->id, false, 2); ?>">
				<?php echo Text::_('COM_GANEWSLETTER_DELETE_ITEM'); ?>
			</a>
		</div>
	</div>

<?php endif; ?>
