<?php
/**
 * @version    0.0.1
 * @package    com_ganewsletter
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\Filesystem\Path;
use \Joomla\CMS\Language;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Ganewsletter\Administrator\Helper\GanewsletterHelper;

/** @var \Joomla\CMS\WebAsset\WebAssetManager $wa */
$this->document->getWebAssetManager()
    ->usePreset('com_ganewsletter.ganewsletterpreset');

// Load admin language file
$lang = Factory::getLanguage();
$lang->load('com_ganewsletter', JPATH_ADMINISTRATOR);

$listOrder  = $this->state->get('list.ordering');
$listDirn   = $this->state->get('list.direction');

$params  = ComponentHelper::getParams('com_ganewsletter');
$newsLoc = $params->get('news_loc', 'newsletters');
$path = Path::clean( Uri::base() . '/images/' . $newsLoc );

$user       = GanewsletterHelper::getSpecificUser();
$canCreate  = $user->authorise('core.create', 'com_ganewsletter');
$canEdit    = $user->authorise('core.edit', 'com_ganewsletter');
$canCheckin = $user->authorise('core.manage', 'com_ganewsletter');
$canChange  = $user->authorise('core.edit.state', 'com_ganewsletter');
$canDelete  = $user->authorise('core.delete', 'com_ganewsletter');
$articleImg = HTMLHelper::cleanImageURL($params->get('rpt_header'));

/*
echo '<pre>Test<br />';
print_r(Factory::getApplication()->getUserState('com_ganewsletter.test.data'));
echo '</pre>';
Factory::getApplication()->setUserState('com_ganewsletter.test.data', null);
        Factory::getApplication()->setUserState('com_ganewsletter.test.data', $imgData);
*/

// Display a heading for the page
if ($this->params->get('page_title', '') > '') {
	echo '<h1>'.$this->params->get('page_title').'</h1>';
} else { 
	echo '<h1>'.Text::_('COM_GANEWSLETTER_TITLE_NEWSLETTERS').'</h1>';
}
?>

<form action="<?php echo htmlspecialchars(Uri::getInstance()->toString()); ?>" method="post"
      name="adminForm" id="adminForm" class="com-content-category__articles">

	<?php echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this)); ?>

    <div class="table-responsive com-contact-categories categories-list">
	<table class="table table-striped" id="newsletterList">
		<thead>
		<tr>
			<?php if (isset($this->items[0]->state)): ?>
				<th width="5%">
					<?php echo HTMLHelper::_('grid.sort', 'JPUBLISHED', 'a.state', $listDirn, $listOrder); ?>
				</th>
			<?php endif; ?>

			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GANEWSLETTER_NEWSLETTERS_TITLE', 'a.title', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GANEWSLETTER_NEWSLETTERS_PUBLISH_DATE', 'a.publish_date', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GANEWSLETTER_CREATED_DATE', 'a.created_date', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GANEWSLETTER_NEWSLETTERS_PDF_GEN', 'a.pdf_gen', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GANEWSLETTER_NEWSLETTERS_ID', 'a.id', $listDirn, $listOrder); ?>
			</th>


			<?php if ($canEdit || $canDelete): ?>
				<th class="center">
					<?php echo Text::_('COM_GANEWSLETTER_ACTIONS'); ?>
				</th>
			<?php endif; ?>

		</tr>
		</thead>
		<tfoot>
		<tr>
			<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
				<?php echo $this->pagination->getListFooter(); ?>
			</td>
		</tr>
		</tfoot>
		<tbody>
		<?php foreach ($this->items as $i => $item) : ?>
			<?php 
                $canEdit = GanewsletterHelper::canUserEdit($user, $item);
                $fileName = str_replace(' ', '_',$item->title);
                $fileName = $newsLoc . '/News-' . $fileName . '.pdf';
            ?>

			<tr class="row<?php echo $i % 2; ?>">

				<?php if (isset($this->items[0]->state)) : ?>
					<?php $class = ($canChange) ? 'active' : 'disabled'; ?>
					<td class="center">
						<a class="btn btn-micro <?php echo $class; ?>" href="<?php echo ($canChange) ? Route::_('index.php?option=com_ganewsletter&task=newsletter.publish&id=' . $item->id . '&state=' . (($item->state + 1) % 2), false, 2) : '#'; ?>">
						<?php if ($item->state == 1): ?>
							<i class="icon-publish"></i>
						<?php else: ?>
							<i class="icon-unpublish"></i>
						<?php endif; ?>
						</a>
					</td>
				<?php endif; ?>

				<td>
					<?php if (isset($item->checked_out) && $item->checked_out) : ?>
						<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'newsletters.', $canCheckin); ?>
					<?php endif; ?>
					<a href="<?php echo Route::_('index.php?option=com_ganewsletter&view=newsletter&id='.(int) $item->id); ?>">
						<?php echo $this->escape($item->title); ?>
					</a>
				</td>
				<td>
					<?php echo $item->publish_date > 0 ? HTMLHelper::_('date', $item->publish_date, Text::_('COM_GANEWSLETTER_DISPLAY_DATE')) : '-'; ?>
				</td>
				<td>
					<?php echo $item->created_date > 0 ? HTMLHelper::_('date', $item->created_date, Text::_('COM_GANEWSLETTER_DISPLAY_DATE')) : '-'; ?>
				</td>
				<td>
                    <?php if($item->pdf_gen) {echo 'Yes';} ?>
                </td>
                <td>
					<?php echo $item->id;?>
				</td>


				<?php if ($canEdit || $canDelete): ?>
					<td class="center">
						<?php if ($canEdit): ?>
							<a href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.edit&id=' . $item->id, false, 2); ?>"
                                class="btn btn-warning" type="button"><i class="icon-edit" ></i>
                            </a>
    						<?php if (\file_exists('images/'.$fileName)): ?>
    							<a href="<?php echo Route::_('images/'.$fileName, false, 2); ?>"
                                    class="btn btn-inverse" target="_blank"><i class="icon-search" ></i>
                                </a>
    							<a href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.sendNews&id=' . $item->id, false, 2); ?>"
                                    class="btn btn-success" title="<?php echo Text::_('COM_GANEWSLETTER_SENDNEWS_MESSAGE'); ?>">
                                    <i class="fas fa-newspaper" ></i>
                                </a>
    						<?php endif; ?>
							<a href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.genPdf&id=' . $item->id, false, 2); ?>"
                                class="btn btn-secondary" type="button">
                                <i class="fas fa-redo" ></i>
                            </a>
						<?php endif; ?>
						<?php if ($canDelete): ?>
							<a href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletter.remove&id=' . $item->id, false, 2); ?>"
                                class="btn btn-danger" type="button"><i class="icon-trash" ></i>
                            </a>
						<?php endif; ?>
					</td>
				<?php endif; ?>

			</tr>
		<?php endforeach; ?>
		</tbody>
	</table>
        </div>
	<?php if ($canCreate) : ?>
		<a href="<?php echo Route::_('index.php?option=com_ganewsletter&task=newsletterform.edit&id=0', false, 0); ?>"
		   class="btn btn-success btn-small"><i class="icon-plus"></i> <?php echo Text::_('COM_GANEWSLETTER_ADD_ITEM'); ?>
        </a>
	<?php endif; ?>

	<input type="hidden" name="task" value=""/>
	<input type="hidden" name="boxchecked" value="0"/>
	<input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
	<input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
	<?php echo HTMLHelper::_('form.token'); ?>
</form>

